﻿# mfence.ps1: restrict mouse cursor within some geometry
# v1.0, 2022-08-27, ed <irc.rizon.net>, MIT-licensed
# https://ocv.me/dev/gist/mfence.ps1
#
# default values assume 3440x1440 monitor, sets fence at 1x1+3440+1436
# blocking access to the top 1px, left 1px, bottom 4px
#
# example usecase:
#   vmware player freaks out and disables your mouse cursor for
#   about 2 seconds if you click the bottom-most row of pixels in
#   fullscreen mode, but setting a fence 4px above it fixes that
#
# ---------------------------------------------------------------------
#
# tip: save the following as "mfence.bat" next to this script:
#   start cmd /c "powershell -command ""set-executionpolicy -scope process bypass; .\mfence.ps1"" & ping -n 2 127.1 >nul"
#
# then create a shortcut to that bat-file and move the shortcut here:
#   %appdata%\Microsoft\Windows\Start Menu\Programs\Startup
#
# and now this will autorun on bootup
#
# alternatively,
#   set-executionpolicy -scope currentuser -executionpolicy unrestricted


Add-Type -TypeDefinition @"
using System;
using System.IO;
using System.Threading;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace A {
  public class B {

    [DllImport("user32.dll")]
    static extern bool ClipCursor(ref RECT lpRect);

    [StructLayout(LayoutKind.Sequential)]
    public struct RECT {
      public int l;
      public int t;
      public int r;
      public int b;
    }
    
    public void Run() {
      Console.WriteLine("[+] hi");

      RECT r;
      r.l = 1;
      r.t = 1;
      r.r = 3440;
      r.b = 1436;

      bool run = true;
      Console.CancelKeyPress += delegate {run=false;};

      bool first = true;
      while (run) {
        ClipCursor(ref r);
        if (first) {
          first = false;
          Console.WriteLine("[+] fence on; {0}x{1},{2}x{3}", r.l, r.t, r.r, r.b);
        }
        System.Threading.Thread.Sleep(300);
      }
      Console.WriteLine("[+] bye");
    }
  }
}
"@ -ReferencedAssemblies System.Windows.Forms

(New-Object -TypeName A.B).Run()
